# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import string


class Formatter(string.Formatter):
    COLORS = {
        'black': '\033[30m',
        'red': '\033[31m',
        'green': '\033[32m',
        'yellow': '\033[33m',
        'blue': '\033[34m',
        'magenta': '\033[35m',
        'cyan': '\033[36m',
        'white': '\033[37m',
        'reset': '\033[0m'
    }

    def __init__(self, width=None):
        self.width = width

    def vformat(self, format_string, args, kwargs, recur=False, width=None):
        result = []
        for literal_text, field_name, format_spec, conversion in self.parse(format_string):
            # output the literal text
            if literal_text:
                result.append(literal_text)

            if format_spec == 'fill':
                recur = True
                result.append('{' + field_name + '}')
                continue

            # if there's a field, output it
            if field_name is not None:
                obj, _ = self.get_field(field_name, args, kwargs)

                # do any conversion on the resulting object
                obj = self.convert_field(obj, conversion)

                # expand the format spec, if needed
                format_spec = self.vformat(format_spec, args, kwargs)

                # format the object and append to the result
                if width is None:
                    field_result = format(obj, format_spec)
                else:
                    field_result = format(obj, str(width))

                result.append(field_result)

        result = ''.join(result)

        if recur:
            return self.vformat(result, args, kwargs, width=self.width - len(result))

        return result

    def get_value(self, key, args, kwargs):
        if isinstance(key, int):
            return args[key]
        elif key in kwargs:
            return kwargs[key]
        elif key in self.COLORS:
            return self.COLORS[key]
        else:
            return '{' + key + '}'



# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import unicode_literals
import sys
import os
import string
import time

from minibar.widgets import Widget
from minibar.formatter import Formatter

__version__ = '0.2.3'
__all__ = ['bar', 'Minibar']


PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3

if PY2:
    text_type = unicode
else:
    text_type = str


def get_terminal_width():
    stty_size = os.popen('stty size', 'r').read()

    if not stty_size:
        # Terminal might not be available, fallback to 80 columns
        return 80
    else:
        rows, columns = stty_size.split()
        return int(columns)


def iprint(string):
    if sys.stdout.isatty():
        print('\r' + string, file=sys.stdout, end='')
        sys.stdout.flush()


def bar(iterator, template='{i}/{total} {bar:fill}', total=None):
    if total is None:
        total = len(iterator)

    minibar = Minibar(total, template)
    for value in minibar.iter(iterator):
        yield value


class Minibar(object):
    def __init__(self, total, template='{i}/{total} {bar:fill}'):
        self.total = total
        self.template = text_type(template)
        self.enabled_widgets = list(self._get_widgets())
        self.terminal_width = get_terminal_width()
        self.fmt = Formatter(self.terminal_width)
        self.start_time = time.time()
        self.counter = 1
        self.render()

    def _get_widgets(self):
        avaliable_widgets = {w.name: w for w in Widget.__subclasses__()}
        for _, field_name, _, _ in string.Formatter().parse(self.template):
            if field_name in avaliable_widgets:
                yield avaliable_widgets[field_name]

    def iter(self, iterator):
        for value in iterator:
            self.render()
            self.counter += 1
            yield value

    def render(self):
        elapsed = time.time() - self.start_time
        kwargs = {w.name: w(self.counter, self.total, elapsed) for w in self.enabled_widgets}
        iprint(self.fmt.format(self.template, **kwargs))

    def inc(self, increment=1):
        self.render()
        self.counter += increment



from setuptools import setup, find_packages
import codecs
import os

import minibar

here = os.path.abspath(os.path.dirname(__file__))

# Get the long description from the relevant file
with codecs.open('README.rst', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="minibar",
    version=minibar.__version__,

    description="Simple text progress bar library",
    long_description=long_description,

    # The project URL.
    url='https://github.com/canassa/minibar',

    # Author details
    author='Cesar Canassa',
    author_email='cesar@canassa.com',

    # Choose your license
    license='MIT',

    classifiers=[
        'Development Status :: 3 - Alpha',

        # Who the project is intended for.
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: User Interfaces',
        'Topic :: Utilities',

        'License :: OSI Approved :: MIT License',

        # Supported Python versions.
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
    ],
    keywords='progressbar progress bar',
    packages=['minibar'],
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from minibar.formatter import Formatter


class Widget(object):
    def __init__(self, counter, total, elapsed):
        self.counter = counter
        self.total = total
        self.elapsed = elapsed


class Bar(Widget):
    name = 'bar'
    template = '{red}[{green}{bar:{bar_width}}{red}]{reset}'
    filler = '▏▎▍▌▋▊▉'

    def __format__(self, width):
        if not width:
            width = 50

        bar_width = int(width) - 2
        tmp = (((self.counter * 100.0) / self.total) * bar_width) / 100
        bar = self.filler[-1] * int(tmp)
        rest = tmp % 1
        if rest:
            bar += self.filler[int((tmp % 1) * len(self.filler))]

        return Formatter().format(self.template, bar=bar, bar_width=bar_width)


class Total(Widget):
    name = 'total'

    def __format__(self, format_spec):
        return format(self.total, format_spec)


class Counter(Widget):
    name = 'i'

    def __format__(self, format_spec):
        return format(self.counter, format_spec)


class Elapsed(Widget):
    name = 'elapsed'

    def __format__(self, format_spec):
        if not format_spec:
            format_spec = '.0f'
        return format(self.elapsed, format_spec)


class ETA(Widget):
    name = 'eta'

    def __format__(self, format_spec):
        eta = int((self.elapsed * (self.total - self.counter)) // self.counter)
        if eta < 60:
            unit = 's'
        elif 60 <= eta < 3600:
            unit = 'm'
            eta = eta // 60
        else:
            unit = 'h'
            eta = eta // 3600

        return format(str(eta) + unit, format_spec)
